app.factory('Characters', ['Balance', 'Utils', 'LocalStorage', 'Skills', 'Player', 'Items', '$injector',
    function (Balance, Utils, LocalStorage, Skills, Player, Items, $injector) {
        var Characters = {};

        var func = {
            loadSave: function () {
                Characters = LocalStorage.getObject('characters', Characters);
            },
            save: function () {
                LocalStorage.setObject('characters', Characters);
            },
            createCharacter: function (name, playerId) {
                if (!name || !playerId) return false;
                var newId = Utils.generateId();
                Characters[newId] = {
                    playerId: playerId,
                    name: name,
                    stats: angular.copy(Balance.getCharacter('starting').stats),
                    freeStatPoints: angular.copy(Balance.getCharacter('starting').statPoints),
                    freeSkillPoints: angular.copy(Balance.getCharacter('starting').skillPoints),
                    id: newId,
                    level: angular.copy(Balance.getCharacter('starting').level),
                    exp: angular.copy(Balance.getCharacter('starting').exp),
                    skills: {},
                    equip: []
                };
                return Characters[newId];
            },
            rebirthRewardById: function (id) {
                return Math.round((Math.pow(1.25, Characters[id].level + 1) - 1) / 100)
            },
            rebirthReward: function () {
                var result = 0;
                Utils.objectEach(Characters, function (id) {
                    result += func.rebirthRewardById(id);
                });
                return result;
            },
            rebirth: function () {
                $injector.get('Battle').stop();
                Player.newQuests(true);
                Player.addLegacy(func.rebirthReward());
                Utils.objectEach(Characters, function (id) {
                    Characters[id].level = 1;
                    Characters[id].equip = [];
                    Characters[id].exp = 0;

                    Characters[id].freeSkillPoints = angular.copy(Balance.getCharacter('starting').skillPoints);
                    Characters[id].skills = {};

                    Characters[id].freeStatPoints = (1 + Player.getCrypt()) * Characters[id].level;
                    Characters[id].stats = angular.copy(Balance.getCharacter('starting').stats);
                });
            },
            addStat: function (id, statId) {
                if (Characters[id].freeStatPoints > 0) {
                    Characters[id].freeStatPoints--;
                    Characters[id].stats[statId]++;
                }
            },
            getStat: function (id, statId) {
                var result = Characters[id].stats[statId] + 1;
                for (var i = 0; i < Characters[id].equip.length; i++) {
                    var itemByToken = Player.getItemByToken(Characters[id].equip[i]);
                    if (itemByToken) {
                        var stat = Items.get()[itemByToken.id].stats[statId];
                        if (stat) {
                            result += stat;
                        }
                    }
                }
                return result
            },
            getSkills: function (id, type) {
                var result = [];
                Utils.objectEach(Characters[id].skills, function (name) {
                    if ((!type || (Skills.getByName(name).type == type)) && (Characters[id].skills[name] > 0)) {
                        result.push(Skills.getByName(name))
                    }
                });
                return result;
            },
            addExp: function (id, exp) {
                Characters[id].exp += Math.round(exp);
                while (Characters[id].exp >= func.getReqExp(id)) {
                    Characters[id].exp -= func.getReqExp(id);
                    func.levelUp(id);
                }
            },
            takeExp: function (id, exp) {
                Characters[id].exp -= exp;
                if (Characters[id].exp < 0) Characters[id].exp = 0;
            },
            levelUp: function (id) {
                Characters[id].level++;
                Player.checkLeveling(Characters[id].name);
                Characters[id].freeSkillPoints += Balance.getCharacter('levelUpSkills');
                Characters[id].freeStatPoints += Balance.getCharacter('levelUpStats') + Player.getCrypt();
            },
            isLearnableSkill: function (id, skillId) {
                var skill = Skills.getByName(skillId);
                if (!skill) return false;
                if (Characters[id].freeSkillPoints < skill.sp_cost) return false;
                if (Characters[id].level < skill.required_level) return false;
                if ((Characters[id].skills[skillId] || 0 ) >= skill.max_level) return false;

                return true
            },
            learnSkill: function (id, skillId) {
                if (func.isLearnableSkill(id, skillId)) {
                    Characters[id].freeSkillPoints -= Skills.getByName(skillId).sp_cost;
                    Characters[id].skills[skillId] = (Characters[id].skills[skillId] + 1) || 1
                }
            },
            getEffect: function (id, effect) {
                var result = [];
                Utils.objectEach(Characters[id].skills, function (name) {
                    if (Skills.getByName(name).effect.indexOf(effect) != -1) {
                        var s = angular.copy(Skills.getByName(name));
                        s.efficiency[effect] *= Characters[id].skills[name];
                        result.push(s)
                    }
                });
                return result
            },
            getHp: function (id) {
                var skills = func.getEffect(id, 'hp');
                var efficiency = 1;
                for (var i = 0; i < skills.length; i++) {
                    efficiency += skills[i].efficiency.hp
                }

                return func.getStat(id, 'vitality') * Balance.getStats().Effect.vitality * efficiency;
            },
            getMp: function (id) {
                var skills = func.getEffect(id, 'mp');
                var efficiency = 1;
                for (var i = 0; i < skills.length; i++) {
                    efficiency += skills[i].efficiency.mp
                }
                return func.getStat(id, 'wisdom') * Balance.getStats().Effect.wisdom * efficiency;
            },
            getAttack: function (id) {
                var attack = 10;
                for (var i = 0; i < Characters[id].equip.length; i++) {
                    if (Player.getItemAttack(Characters[id].equip[i], true)) {
                        attack += Player.getItemAttack(Characters[id].equip[i], true);
                    }
                }
                var skills = func.getEffect(id, 'attack');
                var efficiency = 1;
                for (var i = 0; i < skills.length; i++) {
                    efficiency += skills[i].efficiency.attack
                }
                var a = attack * efficiency;
                return a + Math.round((func.getStat(id, 'agility') * 0.05) * a)
            },
            getDefence: function (id) {
                var defence = 10;
                for (var i = 0; i < Characters[id].equip.length; i++) {
                    if (Player.getItemDefence(Characters[id].equip[i], true)) {
                        defence += Player.getItemDefence(Characters[id].equip[i], true);
                    }
                }
                var skills = func.getEffect(id, 'defence');
                var efficiency = 1;
                for (var i = 0; i < skills.length; i++) {
                    efficiency += skills[i].efficiency.defence
                }
                return defence * efficiency
            },
            getPoints: function () {
                var result = 0;
                /*
                 Utils.objectEach(Characters, function (id) {
                 result += func.getAttack(id) * func.getDefence(id) * (func.getHp(id) + func.getMp(id));
                 });*/
                return result;
            },
            getTaunt: function (id) {
                var taunt = 1;
                var skills = func.getEffect(id, 'taunt');
                for (var i = 0; i < skills.length; i++) {
                    taunt += skills[i].efficiency.taunt
                }

                return taunt
            },
            getEquippedWeapon: function (id) {
                if (!id) return;
                for (var i = 0; i < Characters[id].equip.length; i++) {
                    var token = Characters[id].equip[i];
                    var cItem = Player.getItemByToken(token);
                    if (Items.getWeapons()[cItem.id] != undefined) {
                        return token
                    }
                }
            },
            getEquippedArmor: function (id) {
                if (!id) return;
                for (var i = 0; i < Characters[id].equip.length; i++) {
                    var token = Characters[id].equip[i];
                    var cItem = Player.getItemByToken(token);
                    if (Items.getArmors()[cItem.id] != undefined) {
                        return token
                    }
                }
            },
            getCharacterByToken: function (token) {
                var result;
                Utils.objectEach(Characters, function (id) {
                    if (Characters[id].equip.indexOf(token) != -1) {
                        result = id
                    }
                });
                return result
            },
            equip: function (id, token) {
                func.unEquip(token);
                Characters[id].equip.push(token);
            },
            unEquip: function (token) {
                Utils.objectEach(Characters, function (id) {
                    var index = Characters[id].equip.indexOf(token);
                    if (index != -1) Characters[id].equip.splice(index, 1)

                });
            },
            unEquipWeapon: function (id) {
                func.unEquip(func.getEquippedWeapon(id));
            },
            unEquipArmor: function (id) {
                func.unEquip(func.getEquippedArmor(id));
            },
            getReqExp: function (id) {
                return Math.round(Balance.getCharacter('starting').reqExp
                * Math.pow(Balance.getCharacter('reqExpMultiplier'), Characters[id].level))
            },
            get: function () {
                return Characters;
            },
            toServer: function (p) {
                Utils.objectEach(Characters, function (id) {
                    var Character = Parse.Object.extend("Character");
                    var c = angular.copy(Characters[id]);
                    c.id = c.serverId;
                    var char = new Character();
                    char.set("parent", p);
                    char.save(c, {
                        success: function (data) {
                            Characters[id].serverId = data.id;
                            var query2 = new Parse.Query(Character);
                            query2.greaterThan('level', Characters[id].level);
                            query2.count({
                                success: function (results) {
                                    Characters[id].rank = results;
                                }
                            })
                        }
                    })

                });
            },
            getMaxLevel: function () {
                var max = 1;
                Utils.objectEach(Characters, function (id) {
                    if (Characters[id].level > max) max = Characters[id].level
                });
                return max
            },
            getLength: function () {
                var length = 0;
                Utils.objectEach(Characters, function (id) {
                    length++
                });
                return length;
            },
            getIdByOrder: function (n) {
                var result;
                Utils.objectEach(Characters, function (id) {
                    if (n == 0) result = id;
                    n--;

                });
                return result
            },
            reset: function (id) {
                if (Player.get().crystals >= 5) {
                    Player.takeCrystals(5);
                } else return;
                Characters[id].freeSkillPoints = Characters[id].level;
                Characters[id].skills = {};

                Characters[id].freeStatPoints = (1 + Player.getCrypt()) * Characters[id].level;
                Characters[id].stats = angular.copy(Balance.getCharacter('starting').stats);
            },
            getById: function (id) {
                return Characters[id];
            },
            deleteCharacter: function (id) {
                if (Characters[id] != 'undefined') {
                    return delete Characters[id]
                } else {
                    return false
                }
            }
        };
        return func;
    }
]);