app.factory('Dungeon', ['Player', 'Skills', 'Battle', 'Modal', 'Utils',
    function (Player, Skills, Battle, Modal, Utils) {
        var rewards = [
            {
                item: 'crystal',
                amount: 1
            },
            {
                item: 'crystal',
                amount: 3
            },
            {
                item: 'crystal',
                amount: 5
            },
            {
                item: 'crystal',
                amount: 10
            },
            {
                item: 'exp_crystals',
                amount: 50
            },
            {
                item: 'exp_crystals',
                amount: 200
            },
            {
                item: 'enchant_random',
                amount: 1
            },
            {
                item: 'exp_crystals',
                amount: 1000
            },
            {
                item: 'enchant_random',
                amount: 3
            },
            {
                item: 'exp_crystals',
                amount: 3000
            },
            {
                item: 'enchant_random',
                amount: 5
            },
            {
                item: 'exp_crystals',
                amount: 5000
            },
            {
                item: 'enchant_random',
                amount: 10
            },
            {
                item: 'crystal',
                amount: 50
            },
            {
                item: 'enchant_random',
                amount: 15
            },
            {
                item: 'exp_crystals',
                amount: 10000
            },
            {
                item: 'enchant_random',
                amount: 20
            },
            {
                item: 'enchant_random',
                amount: 25
            },
            {
                item: 'enchant_random',
                amount: 30
            }
        ];

        var func = {
            generateMonster: function (lvl) {
                var monster = {};

                var hp_range = 100 + Math.round(Math.pow(lvl, 2.2) * 50);
                var mp_range = 100 + Math.round(Math.pow(lvl, 1.2) * 20);
                var attack_range = 20 + Math.round(Math.pow(lvl, 1.8) * 10);
                var defence_range = 20 + Math.round(Math.pow(lvl, 1.8) * 10);

                // generate
                monster.hp = Math.round(Math.random() * hp_range);
                monster.mp = Math.round(Math.random() * mp_range);
                monster.attack = Math.round(Math.random() * attack_range);
                monster.defence = Math.round(Math.random() * defence_range);

                // skills
                monster.abilities = {};
                var skillsCount = 0;

                function rndSkill(name) {
                    var isPresent = Math.random() > 0.8;
                    if (isPresent && (skillsCount < 3)) {
                        skillsCount++;
                        monster.abilities[name] = Math.floor(Math.random() * lvl) + 1;
                        var max_lvl = Skills.getByName(name).max_level;
                        if (monster.abilities[name] > max_lvl) monster.abilities[name] = max_lvl;
                    }
                }

                var s = ['fireball', 'heal', 'spirit_strike', 'vampirism'];
                for (var i = 0; i < s.length; i++) rndSkill(s[i]);


                monster.name_id = 'dungeon_monster';
                monster.gold = 0;
                monster.xp = 0;
                monster.drop = [];

                return monster;
            },
            dungeonTime: function () {
                if (!Player.get().dungeonTime) Player.get().dungeonTime = 11111;
                return Math.round((18000000 + Player.get().dungeonTime - (new Date()).getTime()) / 60000)
            },
            resetTime: function () {
                Player.get().dungeonTime = (new Date()).getTime();
            },
            tryDungeon: function () {
                Player.get().dungeonLvl = 1;
                func.resetMonster();
            },
            nextLvl: function () {
                Player.get().dungeonLvl++;
                func.resetMonster();
            },
            sumReward: function () {
                var r = {};
                var l = Math.floor(Player.get().dungeonLvl / 10);
                for (var i = 0; i < l; i++) {
                    if (rewards[i])
                        r[rewards[i].item] = (r[rewards[i].item] || 0) + rewards[i].amount;
                }
                return r;
            },
            resetMonster: function () {
                Player.get().dungeonMonster = func.generateMonster(Player.get().dungeonLvl);
            },
            buyReset: function () {
                if (Player.takeCrystals(1)) func.resetMonster();
            },
            endDungeon: function () {
                Battle.stop();
                var r = func.sumReward();
                Utils.objectEach(r, function (id) {
                    if (id == 'crystal') Player.addCrystals(r[id]);
                    else Player.buyItem(id, r[id], true);
                });

                if (!Player.get().maxDungeon) Player.get().maxDungeon = Player.get().dungeonLvl;
                func.resetTime();
                func.tryDungeon();
            },
            getReward: function (lvl) {
                var n = Math.floor(lvl / 10);
                return rewards[n];
            },
            getRewardLvl: function (lvl) {
                return Math.ceil((lvl + 1) / 10) * 10;
            }
        };

        return func;
    }]);
