app.factory('Player', ['Balance', 'Utils', 'LocalStorage', 'Items', '$injector', 'Monsters',
    function (Balance, Utils, LocalStorage, Items, $injector, Monsters) {
        var Characters;

        var Player = {
            gold: angular.copy(Balance.getPlayer('starting').gold),
            crystals: angular.copy(Balance.getPlayer('starting').crystals),
            characters: [],
            items: [{
                id: 'axe',
                num: 1,
                token: Utils.generateId()
            },
                {
                    id: 'hp_potion',
                    num: 25,
                    token: Utils.generateId()
                }
            ],
            quests: [],
            id: Utils.generateId()
        };

        var selected = {id: 0, m_lvl: '1+'};

        var func = {
            init: function () {
                if (!Characters) Characters = $injector.get('Characters');
            },
            loadSave: function () {
                Player = LocalStorage.getObject('player', Player);
            },
            save: function () {
                LocalStorage.setObject('player', Player);
            },
            getLegacy: function () {
                if (!Player.legacy) Player.legacy = 0;
                return Player.legacy;
            },
            addLegacy: function (l) {
                if (!Player.legacy) Player.legacy = 0;
                if (!Player.allLegacy) Player.allLegacy = 0;

                Player.legacy += l;
                Player.allLegacy += l;
                if (Player.legacy < 0) Player.legacy = 0;
            },
            getStatue: function () {
                if (!Player.statue) Player.statue = 0;
                return Player.statue;
            },
            levelUpStatue: function () {
                if (!Player.statue) Player.statue = 0;

                if (func.getLegacy() >= func.getStatueCost()) {
                    Player.legacy -= func.getStatueCost();
                    Player.statue++;
                }
            },
            getStatueCost: function () {
                return Math.round(10 * Math.pow(1.2, func.getStatue()))
            },
            getCrypt: function () {
                if (!Player.crypt) Player.crypt = 0;
                return Player.crypt;
            },
            levelUpCrypt: function () {
                if (!Player.crypt) Player.crypt = 0;

                if (func.getLegacy() >= func.getCryptCost()) {
                    Player.legacy -= func.getCryptCost();
                    Player.crypt++;

                    Utils.objectEach(Characters.get(), function (id) {
                        Characters.get()[id].freeStatPoints += Characters.get()[id].level;
                    });
                }
            },
            getCryptCost: function () {
                return Math.round(50 * Math.pow(1.5, func.getCrypt()))
            },
            checkKilling: function (id) {
                for (var i = 0; i < Player.quests.length; i++) {
                    if ((Player.quests[i].type == 'q_kill') && (id == Player.quests[i].target)) {
                        Player.quests[i].progress++;
                    }
                }
            },
            checkLeveling: function (name) {
                for (var i = 0; i < Player.quests.length; i++) {
                    if ((Player.quests[i].type == 'q_lvl') && (name == Player.quests[i].target)) {
                        Player.quests[i].progress++;
                    }
                }
            },
            checkHeal: function (hp) {
                for (var i = 0; i < Player.quests.length; i++) {
                    if (Player.quests[i].type == 'q_heal') {
                        Player.quests[i].progress += hp;
                    }
                }
            },
            checkSpendGold: function (g) {
                for (var i = 0; i < Player.quests.length; i++) {
                    if (Player.quests[i].type == 'q_spend') {
                        Player.quests[i].progress += g;
                    }
                }
            },
            checkSpendCrystals: function (c) {
                for (var i = 0; i < Player.quests.length; i++) {
                    if (Player.quests[i].type == 'q_spend_c') {
                        Player.quests[i].progress += c;
                    }
                }
            },
            rewardQuest: function (id) {
                if (func.isQuestAccomplished(id)) {
                    func.addCrystals(Player.quests[id].reward);
                    Player.quests.splice(id, 1)
                }
            },
            isQuestAccomplished: function (id) {
                return Player.quests[id].progress >= Player.quests[id].amount
            },
            questsTime: function () {
                return Math.round((86400000 + Player.questsTime - (new Date()).getTime()) / 3600000)
            },
            newQuests: function (reset) {
                if (Characters.getLength() < 1) return;
                if (reset == undefined) reset = false;
                if (!reset && (((new Date()).getTime() - Player.questsTime) < 86400000)) return;
                Player.quests = [];
                var min = 0;
                var max = 100;
                for (var i = 0; i < 6; i++) {
                    var rnd = Math.floor(min - 0.5 + Math.random() * (max - min + 1));
                    if (rnd < 30) {
                        var maxLvl = Characters.getMaxLevel() - 9;
                        if (maxLvl < 1) maxLvl = 1;
                        var grades = Monsters.getGrades();
                        var g;
                        for (var j = 0; j < grades.length; j++) {
                            if (parseInt(grades[j].name.replace(/\D+/g, ""), 10) <= maxLvl) {
                                g = grades[j].name;
                            }
                        }
                        var monsters = Monsters.getByGrade(g);
                        var m;
                        if (rnd < 4) m = monsters[0];
                        if ((rnd < 8) && (rnd >= 4)) m = monsters[1];
                        if (rnd >= 8) m = monsters[2];

                        var n;
                        var r;
                        if ((rnd % 4) == 0) {
                            n = 25;
                            r = 1;
                        }
                        if ((rnd % 4) == 1) {
                            n = 100;
                            r = 2;
                        }
                        if ((rnd % 4) == 2) {
                            n = 500;
                            r = 3;
                        }
                        if ((rnd % 4) == 3) {
                            n = 1000;
                            r = 5;
                        }
                        Player.quests.push({
                            type: 'q_kill',
                            target: m.name_id,
                            amount: n,
                            reward: r,
                            progress: 0
                        })

                    }
                    if ((rnd > 30) && (rnd <= 55)) {
                        var c;
                        Utils.objectEach(Characters.get(), function (id) {
                            if (!c || (Math.random() >= 0.5)) {
                                c = id;
                            }
                        });
                        var char = Characters.getById(c);
                        var n;
                        var r;
                        if (char.level <= 20) {
                            if ((rnd % 3) == 0) {
                                n = char.level + 3;
                                r = 1;
                            }
                            if ((rnd % 3) == 1) {
                                n = char.level + 5;
                                r = 2;
                            }
                            if ((rnd % 3) == 2) {
                                n = char.level + 7;
                                r = 3;
                            }
                        }
                        if ((char.level > 20) && (char.level <= 35)) {
                            if ((rnd % 3) == 0) {
                                n = char.level + 2;
                                r = 1;
                            }
                            if ((rnd % 3) == 1) {
                                n = char.level + 3;
                                r = 2;
                            }
                            if ((rnd % 3) == 2) {
                                n = char.level + 4;
                                r = 3;
                            }
                        }
                        if ((char.level > 35) && (char.level <= 50)) {
                            if ((rnd % 2) == 0) {
                                n = char.level + 1;
                                r = 1;
                            }
                            if ((rnd % 2) == 1) {
                                n = char.level + 2;
                                r = 2;
                            }
                        }
                        if (char.level > 50) {
                            n = char.level + 1;
                            r = 1;
                        }
                        Player.quests.push({
                            type: 'q_lvl',
                            target: char.name,
                            amount: n,
                            reward: r,
                            progress: char.level
                        })
                    }
                    if ((rnd > 55) && (rnd <= 70)) {
                        var n, r;
                        if (rnd <= 27) {
                            n = 1000;
                            r = 1;
                        }
                        if ((rnd > 27) && (rnd <= 30)) {
                            n = 5000;
                            r = 2;
                        }
                        if ((rnd > 30) && (rnd <= 33)) {
                            n = 15000;
                            r = 3;
                        }
                        if (rnd > 33) {
                            n = 40000;
                            r = 5;
                        }
                        Player.quests.push({
                            type: 'q_heal',
                            amount: n,
                            reward: r,
                            progress: 0
                        })
                    }
                    if ((rnd > 70) && (rnd <= 85)) {
                        var maxLvl = Characters.getMaxLevel();
                        if (maxLvl <= 25) {
                            if (rnd <= 42) {
                                n = 5000;
                                r = 1;
                            }
                            if (rnd > 42) {
                                n = 20000;
                                r = 3;
                            }
                        }
                        if ((maxLvl > 25) && (maxLvl <= 40)) {
                            if (rnd <= 42) {
                                n = 100000;
                                r = 1;
                            }
                            if (rnd > 42) {
                                n = 1000000;
                                r = 5;
                            }
                        }
                        if ((maxLvl > 40) && (maxLvl <= 65)) {
                            if (rnd <= 42) {
                                n = 10000000;
                                r = 1;
                            }
                            if (rnd > 42) {
                                n = 50000000;
                                r = 3;
                            }
                        }
                        if (maxLvl > 65) {
                            n = 100000000;
                            r = 1
                        }
                        Player.quests.push({
                            type: 'q_spend',
                            amount: n,
                            reward: r,
                            progress: 0
                        })
                    }
                    if (rnd > 85) {
                        var maxLvl = Characters.getMaxLevel();
                        if (maxLvl <= 25) {
                            if (rnd <= 42) {
                                n = 5;
                                r = 2;
                            }
                            if (rnd > 42) {
                                n = 10;
                                r = 5;
                            }
                        }
                        if ((maxLvl > 25) && (maxLvl <= 40)) {
                            if (rnd <= 42) {
                                n = 20;
                                r = 10;
                            }
                            if (rnd > 42) {
                                n = 30;
                                r = 15;
                            }
                        }
                        if ((maxLvl > 40) && (maxLvl <= 65)) {
                            if (rnd <= 42) {
                                n = 50;
                                r = 30;
                            }
                            if (rnd > 42) {
                                n = 100;
                                r = 60;
                            }
                        }
                        if (maxLvl > 65) {
                            n = 200;
                            r = 150;
                        }
                        Player.quests.push({
                            type: 'q_spend_c',
                            amount: n,
                            reward: r,
                            progress: 0
                        })
                    }
                }
                Player.questsTime = (new Date()).getTime();
            },
            addGold: function (g) {
                if (g < 0) return false;
                return Player.gold += g;
            },
            takeGold: function (g) {
                if (g < 0) return false;
                if (g > Player.gold) {
                    return false
                } else {
                    func.checkSpendGold(g);
                    Player.gold -= g;
                    return true
                }
            },
            addCrystals: function (g) {
                return Player.crystals += g;
            },
            takeCrystals: function (g) {
                if (g > Player.crystals) {
                    return false
                } else {
                    func.checkSpendCrystals(g);
                    Player.crystals -= g;
                    return true
                }
            },
            getItemDefence: function (id, token) {
                if (token) {
                    var item = func.getItemByToken(id)
                } else {
                    var item = Player.items[id];
                }
                var enchant = item.enchant || 0;
                var defence = Items.get()[item.id].defence;
                if (!defence) return false;
                return Math.round(defence * Math.pow(1 + Balance.getItems('enchantEfficiency'), enchant))
            },
            getItemAttack: function (id, token) {
                if (token) {
                    var item = func.getItemByToken(id)
                } else {
                    var item = Player.items[id];
                }
                var enchant = item.enchant || 0;
                var attack = Items.get()[item.id].attack;
                if (!attack) return false;
                return Math.round(attack * Math.pow(1 + Balance.getItems('enchantEfficiency'), enchant))
            },
            getItemByToken: function (token) {
                for (var i = 0; i < Player.items.length; i++) {
                    if (Player.items[i].token == token) return Player.items[i]
                }
            },
            findItem: function (itemId, isIds) {
                var foundedItems = [];
                for (var i = 0; i < Player.items.length; i++) {
                    if (Player.items[i].id == itemId) {
                        if (!isIds) {
                            foundedItems.push(Player.items[i]);
                        } else {
                            foundedItems.push(i)
                        }
                    }
                }
                return foundedItems;
            },
            deleteItem: function (itemId, num) {
                var num = num || 1;
                if (Player.items[itemId].num > num) {
                    Player.items[itemId].num -= num
                } else {
                    if ((itemId < selected.id) || (((itemId + 1) == Player.items.length)) && itemId != 0) selected.id--;

                    Characters.unEquip(Player.items[itemId].token);
                    Player.items.splice(itemId, 1);
                }
            },
            isEnchantable: function (itemId) {
                var name = Player.items[itemId].id;
                if (!Items.getByName(name).enchantable) return false;
                var enchantScroll = 'enchant_' + Items.getByName(name).grade;
                return ((func.findItem(enchantScroll) || [] ).length > 0)
            },
            enchantItem: function (itemId) {
                var item = Player.items[itemId];
                if (func.isEnchantable(itemId)) {
                    var success = false;
                    if (Player.items[itemId].enchant >= 3) {
                        var min = 0;
                        var max = 100;
                        var rnd = min - 0.5 + Math.random() * (max - min + 1);
                        if (rnd > 50) success = true;
                    } else success = true;
                    if (success) {
                        Player.items[itemId].enchant = (Player.items[itemId].enchant || 0) + 1;
                    } else {
                        func.deleteItem(itemId, 1);
                    }

                    var scrolls = func.findItem('enchant_' + Items.getByName(item.id).grade, true);
                    func.deleteItem(scrolls[0])
                }
            },
            openChest: function (itemId) {
                var item = Player.items[itemId];
                var min = 0;
                var max = 100;
                var rnd = min - 0.5 + Math.random() * (max - min + 1);
                if (rnd <= 5) {
                    func.buyItem('enchant_ma', 1, true)
                }
                if ((rnd > 5) && (rnd <= 20)) {
                    func.buyItem('enchant_el', 1, true)
                }
                if ((rnd > 20) && (rnd <= 50)) {
                    func.buyItem('enchant_ex', 1, true)
                }
                if (rnd > 50) {
                    func.buyItem('enchant_ba', 1, true)
                }
                func.deleteItem(itemId, 1);
            },
            buyItem: function (itemId, num, free) {
                var item = Items.getByName(itemId);
                var cost = item.cost;
                var num = num || 1;
                var currency = 'gold';
                if (item.crystals) currency = 'crystals';
                if (free || ((cost * num) <= Player[currency])) {
                    var presented = false;
                    if (Items.getByName(itemId).stackable) {
                        for (var i = 0; i < Player.items.length; i++) {
                            if (Player.items[i].id == itemId) {
                                presented = true;
                                Player.items[i].num += num
                            }
                        }
                    }
                    if (!presented) {
                        Player.items.push({
                            id: itemId,
                            num: num,
                            token: Utils.generateId()
                        });
                    }
                    if (!free) {
                        if (currency == 'gold') func.takeGold(cost * num);
                        if (currency == 'crystals') func.takeCrystals(cost * num);
                    }
                }
            },
            getSellCost: function (id, num) {
                var num = num || 1;
                var cost = Items.getByName(Player.items[id].id).cost;
                cost *= Balance.getItems('sellCost');
                cost *= Math.pow(1 + Balance.getItems('enchantCost'), Player.items[id].enchant || 0);
                cost = Math.round(cost);
                cost *= num;
                return cost
            },
            sellItem: function (id, num) {
                var num = num || 1;
                if (!Player.items[id]) return;

                var cost = func.getSellCost(id, num);
                func.addGold(cost);
                func.deleteItem(id, num)
            },
            getWeapons: function () {
                var result = [];
                for (var i = 0; i < Player.items.length; i++) {
                    if (Items.getWeapons()[Player.items[i].id] != undefined) {
                        result.push(Player.items[i].token)
                    }
                }
                return result
            },
            getArmors: function () {
                var result = [];
                for (var i = 0; i < Player.items.length; i++) {
                    if (Items.getArmors()[Player.items[i].id] != undefined) {
                        result.push(Player.items[i].token)
                    }
                }
                return result
            },
            setName: function (name) {
                if (name) Player.name = name;
            },
            addCharacter: function (id) {
                if (id) Player.characters.push(id);
                return Player.characters;
            },
            deleteCharacter: function (id) {
                if (id) Player.characters.splice(Player.characters.indexOf(id), 1);
                return Player.characters;
            },
            getCharacters: function () {
                return Player.characters;
            },
            getSelected: function () {
                return selected;
            },
            get: function () {
                return Player;
            }
        };

        return func

    }]);